import React from 'react';
import isEqual from '@guanghechen/fast-deep-equal';

function useDeepCompareCallback(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps))
        signal.current += 1;
    prevDeps.current = deps;
    return React.useCallback(fn, [signal.current]);
}

function useRefreshRef(value) {
    const ref = React.useRef(value);
    ref.current = value;
    return ref;
}

function useBeforeUnloadEffect(fn, deps) {
    const beforeunload = useDeepCompareCallback(fn, deps);
    React.useLayoutEffect(() => {
        window.addEventListener('beforeunload', beforeunload);
        return () => window.removeEventListener('beforeunload', beforeunload);
    }, [beforeunload]);
}
function useBeforeUnloadAsyncEffect(fn, deps, preventDefault = true) {
    const preventDefaultRef = useRefreshRef(preventDefault);
    const beforeunload = useDeepCompareCallback(async (event) => {
        if (preventDefaultRef.current === false) {
            return fn(event);
        }
        event.preventDefault();
        try {
            await fn(event);
        }
        finally {
            delete event.returnValue;
        }
    }, deps);
    React.useLayoutEffect(() => {
        window.addEventListener('beforeunload', beforeunload);
        return () => window.removeEventListener('beforeunload', beforeunload);
    }, [beforeunload]);
}

function useDeepCompareEffect(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps))
        signal.current += 1;
    prevDeps.current = deps;
    React.useEffect(fn, [signal.current]);
}

function useDeepCompareMemo(fn, deps) {
    const signal = React.useRef(0);
    const prevDeps = React.useRef(deps);
    if (!isEqual(prevDeps.current, deps))
        signal.current += 1;
    prevDeps.current = deps;
    return React.useMemo(fn, [signal.current]);
}

function useEventCallback(handler) {
    const handlerRef = React.useRef(handler);
    React.useLayoutEffect(() => {
        handlerRef.current = handler;
    });
    return React.useCallback((...args) => {
        const handle = handlerRef.current;
        return handle(...args);
    }, []);
}

function useInterval(callback, duration) {
    const callbackRef = React.useRef(callback);
    React.useEffect(() => {
        callbackRef.current = callback;
    }, [callback]);
    React.useEffect(() => {
        const tick = () => {
            if (callbackRef.current === undefined)
                return;
            callbackRef.current();
        };
        const intervalId = setInterval(tick, duration);
        return () => clearInterval(intervalId);
    }, [duration]);
}

function usePreviousState(value) {
    const ref = React.useRef(value);
    React.useEffect(() => {
        ref.current = value;
    }, [value]);
    return ref.current;
}

function useSyncState(initialState) {
    const [state, setState] = React.useState(initialState);
    React.useEffect(() => setState(initialState), [initialState]);
    return [state, setState];
}

export { useBeforeUnloadAsyncEffect, useBeforeUnloadEffect, useDeepCompareCallback, useDeepCompareEffect, useDeepCompareMemo, useEventCallback, useInterval, usePreviousState, useRefreshRef, useSyncState };
